@@ -151,11 +151,11 @@ static int conf_set_sym_val(struct symbol *sym, int def, int def_flags, char *p)
 	return 0;
 }
 
-int conf_read_simple(const char *name, int def)
+int conf_read_simple_level(const char *name, int def, struct conf_level *l)
 {
 	FILE *in = NULL;
 	char line[1024];
-	char *p, *p2;
+	char *p, *p2, *lp;
 	struct symbol *sym;
 	int i, def_flags;
 
@@ -199,7 +199,7 @@ load:
 	conf_unsaved = 0;
 
 	def_flags = SYMBOL_DEF << def;
-	for_all_symbols(i, sym) {
+	for_all_symbols_level(i, sym, l) {
 		sym->flags |= SYMBOL_CHANGED;
 		sym->flags &= ~(def_flags|SYMBOL_VALID);
 		if (sym_is_choice(sym))
@@ -207,6 +207,7 @@ load:
 		switch (sym->type) {
 		case S_INT:
 		case S_HEX:
+		case S_EXECUTE:
 		case S_STRING:
 			if (sym->def[def].val)
 				free(sym->def[def].val);
@@ -229,14 +230,20 @@ load:
 			*p++ = 0;
 			if (strncmp(p, "is not set", 10))
 				continue;
+			lp = line + 2;
+			if (l
+			    && l->conf_prefix
+			    && strstr(lp, l->conf_prefix) == lp) {
+				lp += strlen(l->conf_prefix);
+			}
 			if (def == S_DEF_USER) {
-				sym = sym_find(line + 2);
+				sym = sym_find_level(lp, l);
 				if (!sym) {
 					sym_add_change_count(1);
 					break;
 				}
 			} else {
-				sym = sym_lookup(line + 2, 0);
+				sym = sym_lookup_level(lp, 0, l);
 				if (sym->type == S_UNKNOWN)
 					sym->type = S_BOOLEAN;
 			}
@@ -264,14 +271,20 @@ load:
 				if (*p2 == '\r')
 					*p2 = 0;
 			}
+			lp = line;
+			if (l
+			    && l->conf_prefix
+			    && strstr(lp, l->conf_prefix) == lp) {
+				lp += strlen(l->conf_prefix);
+			}
 			if (def == S_DEF_USER) {
-				sym = sym_find(line);
+				sym = sym_find_level(lp, l);
 				if (!sym) {
 					sym_add_change_count(1);
 					break;
 				}
 			} else {
-				sym = sym_lookup(line, 0);
+				sym = sym_lookup_level(lp, 0, l);
 				if (sym->type == S_UNKNOWN)
 					sym->type = S_OTHER;
 			}
@@ -315,7 +328,7 @@ load:
 	return 0;
 }
 
-int conf_read(const char *name)
+int conf_read_level(const char *name, struct conf_level *l)
 {
 	struct symbol *sym, *choice_sym;
 	struct property *prop;
@@ -324,10 +337,10 @@ int conf_read(const char *name)
 
 	sym_set_change_count(0);
 
-	if (conf_read_simple(name, S_DEF_USER))
+	if (conf_read_simple_level(name, S_DEF_USER, l))
 		return 1;
 
-	for_all_symbols(i, sym) {
+	for_all_symbols_level(i, sym, l) {
 		sym_calc_value(sym);
 		if (sym_is_choice(sym) || (sym->flags & SYMBOL_AUTO))
 			goto sym_ok;
@@ -364,7 +377,7 @@ int conf_read(const char *name)
 		sym->flags &= flags | ~SYMBOL_DEF_USER;
 	}
 
-	for_all_symbols(i, sym) {
+	for_all_symbols_level(i, sym, l) {
 		if (sym_has_value(sym) && !sym_is_choice_value(sym)) {
 			/* Reset values of generates values, so they'll appear
 			 * as new, if they should become visible, but that
@@ -374,6 +387,7 @@ int conf_read(const char *name)
 			if (sym->visible == no && !conf_unsaved)
 				sym->flags &= ~SYMBOL_DEF_USER;
 			switch (sym->type) {
+			case S_EXECUTE:
 			case S_STRING:
 			case S_INT:
 			case S_HEX:
@@ -394,15 +408,42 @@ int conf_read(const char *name)
 	return 0;
 }
 
+int conf_read(const char *name) {
+	int r = conf_read_level(name, 0);
+	if (!r && !name) {
+		struct conf_level *l = conf_levels;
+		while (l) {
+			char *n = resolve_vars(l->conf, l->parent);
+			if (n) {
+				if (r = conf_read_level(n, l))
+					break;
+				free(n);
+			}
+			l = l->n;
+		}
+	} 
+	return r;
+}
+
+int conf_reload(struct symbol *sym, struct conf_level *l) {
+	int r = 0;
+	if (l) {
+		char *n = resolve_vars(l->conf, l->parent);
+		r = conf_read_level(n, l);
+		free(n);
+	}
+	return r;
+}
+
 /* Write a S_STRING */
-static void conf_write_string(bool headerfile, const char *name,
+static void conf_write_string(bool headerfile, const char *prefix, const char *name,
                               const char *str, FILE *out)
 {
 	int l;
 	if (headerfile)
-		fprintf(out, "#define %s \"", name);
+		fprintf(out, "#define %s%s \"", prefix, name);
 	else
-		fprintf(out, "%s=\"", name);
+		fprintf(out, "%s%s=\"", prefix, name);
 
 	while (1) {
 		l = strcspn(str, "\"\\");
@@ -417,6 +458,8 @@ static void conf_write_string(bool headerfile, const char *name,
 	fputs("\"\n", out);
 }
 
+#define CONFPREFIX(sym) ((sym && sym->level && sym->level->conf_prefix) ? sym->level->conf_prefix : "")
+
 static void conf_write_symbol(struct symbol *sym, enum symbol_type type,
                               FILE *out, bool write_no)
 {
@@ -428,23 +471,23 @@ static void conf_write_symbol(struct symbol *sym, enum symbol_type type,
 		switch (sym_get_tristate_value(sym)) {
 		case no:
 			if (write_no)
-				fprintf(out, "# %s is not set\n", sym->name);
+				fprintf(out, "# %s%s is not set\n", CONFPREFIX(sym), sym->name);
 			break;
 		case mod:
-			fprintf(out, "%s=m\n", sym->name);
+			fprintf(out, "%s%s=m\n", CONFPREFIX(sym), sym->name);
 			break;
 		case yes:
-			fprintf(out, "%s=y\n", sym->name);
+			fprintf(out, "%s%s=y\n", CONFPREFIX(sym), sym->name);
 			break;
 		}
 		break;
 	case S_STRING:
-		conf_write_string(false, sym->name, sym_get_string_value(sym), out);
+		conf_write_string(false, CONFPREFIX(sym), sym->name, sym_get_string_value(sym), out);
 		break;
 	case S_HEX:
 	case S_INT:
 		str = sym_get_string_value(sym);
-		fprintf(out, "%s=%s\n", sym->name, str);
+		fprintf(out, "%s%s=%s\n", CONFPREFIX(sym), sym->name, str);
 		break;
 	case S_OTHER:
 	case S_UNKNOWN:
@@ -527,7 +570,7 @@ next_menu:
 	return 0;
 }
 
-int conf_write(const char *name)
+int conf_write_level(const char *name, struct conf_level *l)
 {
 	FILE *out;
 	struct symbol *sym;
@@ -598,7 +641,7 @@ int conf_write(const char *name)
 	while (menu) {
 		sym = menu->sym;
 		if (!sym) {
-			if (!menu_is_visible(menu))
+			if (!menu_is_visible(menu) || menu->level != l)
 				goto next;
 			str = menu_get_prompt(menu);
 			fprintf(out, "\n"
@@ -607,7 +650,7 @@ int conf_write(const char *name)
 				     "#\n", str);
 		} else if (!(sym->flags & SYMBOL_CHOICE)) {
 			sym_calc_value(sym);
-			if (!(sym->flags & SYMBOL_WRITE))
+			if (sym->level != l || !(sym->flags & SYMBOL_WRITE))
 				goto next;
 			sym->flags &= ~SYMBOL_WRITE;
 			type = sym->type;
@@ -653,6 +696,22 @@ next:
 	return 0;
 }
 
+int conf_write(const char *name) {
+	int r = conf_write_level(name, 0);
+	if (!r && !name) {
+		struct conf_level *l = conf_levels;
+		while (l) {
+			char *n = resolve_vars(l->conf, l->parent);
+			if (r = conf_write_level(n, l))
+				break;
+			l = l->n;
+			if (n)
+				free(n);
+		}
+	} 
+	return r;
+}
+
 static int conf_split_config(void)
 {
 	const char *name;
@@ -664,7 +723,7 @@ static int conf_split_config(void)
 	int res, i, fd;
 
 	name = conf_get_autoconfig_name();
-	conf_read_simple(name, S_DEF_AUTO);
+	conf_read_simple_level(name, S_DEF_AUTO, 0);
 
 	opwd = malloc(256);
 	_name = strdup(name);
@@ -871,7 +930,7 @@ int conf_write_autoconf(void)
 			}
 			break;
 		case S_STRING:
-			conf_write_string(true, sym->name, sym_get_string_value(sym), out_h);
+			conf_write_string(true, CONFPREFIX(sym), sym->name, sym_get_string_value(sym), out_h);
 			break;
 		case S_HEX:
 			str = sym_get_string_value(sym);
